package com.pointrlabs.sample.selection


import com.pointrlabs.core.management.models.Building
import com.pointrlabs.sample.selection.BuildingSelectionAdapter.BuildingItemAdapter
import android.view.ViewGroup
import android.view.LayoutInflater
import android.view.View
import com.pointrlabs.sample.R
import androidx.recyclerview.widget.RecyclerView
import android.widget.TextView
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter

/**
 * List adapter for venue selection list
 */
class BuildingSelectionAdapter internal constructor(private val listener: Listener) :
    ListAdapter<Building, BuildingItemAdapter>(BuildingItemCallback()) {
    /**
     * Listener interface to advertise events of Venue item rows
     */
    interface Listener {
        /**
         * Invoked when user selects a venue
         *
         * @param buildingInternalIdentifier of [Building] that's selected
         */
        fun onBuildingSelected(buildingInternalIdentifier: Int)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BuildingItemAdapter {
        val inflater = LayoutInflater.from(parent.context)
        val venueItemView = inflater.inflate(R.layout.item_venue, parent, false)
        return BuildingItemAdapter(venueItemView, listener)
    }

    override fun onBindViewHolder(holder: BuildingItemAdapter, position: Int) {
        holder.bind(getItem(position)!!)
    }

    class BuildingItemAdapter(itemView: View, listener: Listener) :
        RecyclerView.ViewHolder(itemView) {
        private val venueTitleTextView: TextView
        private var building: Building? = null

        init {
            venueTitleTextView = itemView.findViewById(R.id.titleTextView)
            itemView.setOnClickListener {
                listener.onBuildingSelected(
                    building!!.internalIdentifier
                )
            }
        }

        fun bind(building: Building) {
            this.building = building
            venueTitleTextView.text = building.title
        }
    }

    private class BuildingItemCallback : DiffUtil.ItemCallback<Building>() {
        override fun areItemsTheSame(oldItem: Building, newItem: Building): Boolean {
            return oldItem == newItem
        }

        override fun areContentsTheSame(oldItem: Building, newItem: Building): Boolean {
            return oldItem == newItem
        }
    }
}